#!/usr/bin/python3
# -*- coding: iso-8859-1 -*-
#
__author__="atareao"
__date__ ="$22-jan-2012$"
#
#
# Copyright (C) 2012 Lorenzo Carbonell
# lorenzo.carbonell.cerezo@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
import sys
import os
import logging

if os.path.exists('/tmp/nemo_pushbullet_temporal_logging.log'):
	os.remove('/tmp/nemo_pushbullet_temporal_logging.log')
logger = logging.getLogger('nemo_pushbullet')
hdlr = logging.FileHandler('/tmp/nemo_pushbullet_temporal_logging.log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr) 
logger.setLevel(logging.INFO)

try:
	userlib = '/opt/extras.ubuntu.com/pushbullet-commons/share/pushbullet-commons'
	if os.path.exists(userlib) and userlib not in sys.path:		
		sys.path.append('/opt/extras.ubuntu.com/pushbullet-commons/share/pushbullet-commons')
		import commons_comun
		from commons_configurator import Configuration as TokenConfiguration
		from pushbullet_dialogs import SendToDialog
		from logindialog import LoginDialog
except Exception as e:
	logger.info(e)

from gi.repository import Nemo as FileManager
from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Gdk

import json
import dbus
import urllib
from dbus.mainloop.glib import DBusGMainLoop
from os.path import basename

import locale
import gettext

LANGDIR = '/usr/share/locale-langpack'
APP = 'nemo-pushbullet'
ICON = 'pushbullet'
VERSION = '0.3.0-0extras14.04.0'
SEPARATOR = u'\u2015' * 10
URL = 'https://www.pushbullet.com/authorize?client_id=RpkMU0UUoiDMNZnY1ZLJXWLw7QYm4QRY&redirect_uri=http%3A%2F%2Fwww.atareao.es%2Ftag%2Fpushbullet-indicator&response_type=token&scope=everything'

locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(APP, LANGDIR)
gettext.textdomain(APP)
_ = gettext.gettext	

# Callbacks for asynchronous calls

failed = False
pushbullet_replied = False
raise_replied = False

def handle_pushbullet_reply():
	global pushbullet_replied
	pushbullet_replied = True
	if pushbullet_replied and raise_replied:
		loop.quit()

def handle_pushbullet_error(e):
	global failed
	global pushbullet_replied
	pushbullet_replied = True
	failed = True
	print("HelloWorld raised an exception! That's not meant to happen...")
	print("\t", str(e))
	if pushbullet_replied and raise_replied:
		loop.quit()

def handle_raise_reply():
	global failed
	global raise_replied
	raise_replied = True
	failed = True
	print("RaiseException returned normally! That's not meant to happen...")
	if pushbullet_replied and raise_replied:
		loop.quit()

def handle_raise_error(e):
	global raise_replied
	raise_replied = True
	print("RaiseException raised an exception as expected:")
	print("\t", str(e))

def get_files(files_in):
	files = []
	for file_in in files_in:		
		file_in = urllib.unquote(file_in.get_uri()[7:])
		if os.path.isfile(file_in):
			files.append(file_in)
	return files

dbus_loop = DBusGMainLoop(set_as_default=True)
bus = dbus.SessionBus(mainloop=dbus_loop)

"""
Tools to send files with Pushbullet
"""	
class PushbulletMenuProvider(GObject.GObject, FileManager.MenuProvider):
	"""Implements the 'Replace in Filenames' extension to the File Manager right-click menu"""

	def __init__(self):
		"""File Manager crashes if a plugin doesn't implement the __init__ method"""
		pass
		
	def about(self,menu):
		ad=Gtk.AboutDialog()
		ad.set_name(APP)
		ad.set_icon_name(ICON)
		ad.set_version(VERSION)
		ad.set_copyright('Copyrignt (c) 2014-2015\nLorenzo Carbonell')
		ad.set_comments(_('Tools to send files with Pushbullet'))
		ad.set_license(''+
		'This program is free software: you can redistribute it and/or modify it\n'+
		'under the terms of the GNU General Public License as published by the\n'+
		'Free Software Foundation, either version 3 of the License, or (at your option)\n'+
		'any later version.\n\n'+
		'This program is distributed in the hope that it will be useful, but\n'+
		'WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n'+
		'or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for\n'+
		'more details.\n\n'+
		'You should have received a copy of the GNU General Public License along with\n'+
		'this program.  If not, see <http://www.gnu.org/licenses/>.')
		ad.set_website('http://www.atareao.es')
		ad.set_website_label('http://www.atareao.es')
		ad.set_authors(['Lorenzo Carbonell <lorenzo.carbonell.cerezo@gmail.com>'])
		ad.set_documenters(['Lorenzo Carbonell <lorenzo.carbonell.cerezo@gmail.com>'])
		ad.set_program_name(APP)
		ad.set_logo_icon_name(ICON)
		ad.run()
		ad.destroy()		
	def on_init_clicked(self,widget):
		try:
			print('****************************************************')
			print('register pushbullet service')
			pushBulletService = bus.get_object('es.atareao.pushbullet','/es/atareao/pushbullet')
			print('****************************************************')
			tokenConfiguration = TokenConfiguration()
			if pushBulletService.isLogin() == False:
				ld = LoginDialog()
				ld.run()
				oauth_access_token = ld.code
				tokenConfiguration.set('oauth_access_token',oauth_access_token)
				tokenConfiguration.save()
				pushbulletService.restart()
		except dbus.DBusException as e:
			print(e)

	def on_send_push_clicked(self,widget,selected):
		try:
			print('****************************************************')
			print('register pushbullet service')
			pushBulletService = bus.get_object('es.atareao.pushbullet','/es/atareao/pushbullet')
			print('****************************************************')
			idevices = json.loads(pushBulletService.getDevices())
			icontacts = json.loads(pushBulletService.getContacts())
			tdevices = []
			tcontacts = []
			logger.info(str(idevices))
			for device in idevices:
				if 'nickname' in idevices[device].keys() and len(idevices[device]['nickname'])>0:
					label = idevices[device]['nickname']
					tdevices.append({'device_iden':device,'label':label})
				elif 'model' in idevices[device].keys() and len(idevices[device]['model'])>0:
					label = idevices[device]['model']
					tdevices.append({'device_iden':device,'label':label})
			sdevices = sorted(tdevices, key=lambda k: k['label']) 
			for contact in icontacts:
				if icontacts[contact]['active'] and 'name' in icontacts[contact].keys() and 'email' in icontacts[contact].keys():
					tcontacts.append({'contact_iden':contact,'name':icontacts[contact]['name'],'email':icontacts[contact]['email']})
			scontacts = sorted(tcontacts, key=lambda k: k['name']) 
			sdevices = sorted(tdevices, key=lambda k: k['label']) 				
			cm = SendToDialog(sdevices,scontacts)
			if 	cm.run() == Gtk.ResponseType.ACCEPT:
					cm.hide()
					logger.info(cm.get_response())
					ans = cm.get_response()
					label = ans['label']
					iden = ans['iden']
					ttype = ans['type']
					logger.info('*******************************************************')
					logger.info('Upload files')
					logger.info(selected)
					files = get_files(selected)
					logger.info(files)
					logger.info('*******************************************************')
					for filename in files:
						logger.info('-*=1=*-')
						logger.info(label)
						logger.info(iden)
						logger.info(ttype)
						logger.info(filename)
						pushBulletService.send_file(iden,filename, '', ttype,
									dbus_interface='es.atareao.pushbullet',
									reply_handler=handle_pushbullet_reply,
									error_handler=handle_pushbullet_error)
						logger.info('-*=2=*-')		
			cm.destroy()
		except dbus.DBusException as e:
			logger.info(e)
		except Exception as e1:
			logger.info(e1)
	def get_file_items(self, window, sel_items):
		"""Adds the 'Replace in Filenames' menu item to the File Manager right-click menu,
		   connects its 'activate' signal to the 'run' method passing the selected Directory/File"""
		top_menuitem = FileManager.MenuItem(name='PushbulletMenuProvider::Gtk-pdf-tools',
								 label=_('Pushbullet'),
								 tip=_('Send files with Pushbullet'),
								 icon='Gtk-find-and-replace')
		#
		submenu = FileManager.Menu()
		top_menuitem.set_submenu(submenu)
		try:
			print('****************************************************')
			print('register pushbullet service')
			pushBulletService = bus.get_object('es.atareao.pushbullet','/es/atareao/pushbullet')
			print('****************************************************')			
			if pushBulletService.is_login():
				sub_menuitem_95 = FileManager.MenuItem(name='PushbulletMenuProvider::Gtk-pushbullet-95',
										 label=_('Send to ...'),
										 tip=_('Send to ...'),
										 icon='Gtk-find-and-replace')
				sub_menuitem_95.connect('activate', self.on_send_push_clicked, sel_items)
				submenu.append_item(sub_menuitem_95)
			else:
				sub_menuitem_97 = FileManager.MenuItem(name='PushbulletMenuProvider::Gtk-pushbullet-97',
										 label=_('Init Pushbullet'),
										 tip=_('Init Pushbullet'),
										 icon='Gtk-find-and-replace')
				sub_menuitem_97.connect('activate', self.on_init_clicked)
				submenu.append_item(sub_menuitem_97)
		except dbus.DBusException as e:
			print(e)

		#		
		sub_menuitem_98 = FileManager.MenuItem(name='PushbulletMenuProvider::Gtk-None-02',
								 label=SEPARATOR)
		submenu.append_item(sub_menuitem_98)
		#		
		sub_menuitem_99 = FileManager.MenuItem(name='PushbulletMenuProvider::Gtk-pushbullet-99',
								 label=_('About'),
								 tip=_('About'),
								 icon='Gtk-find-and-replace')
		sub_menuitem_99.connect('activate', self.about)
		submenu.append_item(sub_menuitem_99)
		#		
		return top_menuitem,

if __name__ == '__main__':
	pbmp = PushbulletMenuProvider()
	print(pbmp.get_file_items(None,[]))
	pass
